# Stubs for handler (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.
from typing import Any, Callable, Union

from sanic.request import Request
from sanic.response import HTTPResponse

HandlerSignature = Callable[[Request, Exception], Union[HTTPResponse, Any]]

class Handler:
    default_message_key: str = ...

    no_authorization: HandlerSignature = ...
    expired_signature: HandlerSignature = ...
    invalid_header: HandlerSignature = ...
    invalid_token: HandlerSignature = ...
    jwt_decode_error: HandlerSignature = ...
    wrong_token: HandlerSignature = ...
    revoked_token: HandlerSignature = ...
    fresh_token_required: HandlerSignature = ...
    access_denied: HandlerSignature = ...
