# Stubs for config (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from datetime import timedelta
from typing import Any, Optional, Tuple, Type, Union

from sanic_jwt_extended.blacklist import BlacklistABC

class Config:
    read_only: bool = ...
    secret_key: Optional[str] = ...
    public_key: Optional[str] = ...
    private_key: Optional[str] = ...
    default_iss: Optional[str] = ...
    default_aud: Optional[str] = ...
    json_encoder: Any = ...
    token_location: Tuple[str] = ...
    access_token_expires: Union[timedelta, bool] = ...
    refresh_token_expires: Union[timedelta, bool] = ...
    algorithm: str = ...
    public_claim_namespace: str = ...
    private_claim_prefix: str = ...
    jwt_header_key: str = ...
    jwt_header_prefix: str = ...
    jwt_query_param_name: str = ...
    use_acl: bool = ...
    acl_claim: str = ...
    use_blacklist: bool = ...
    blacklist_class: Optional[Type[BlacklistABC]] = ...
    def __setattr__(self, key: Any, value: Any) -> None: ...
    def __init__(
        self,
        read_only: Optional[str] = ...,
        secret_key: Optional[str] = ...,
        public_key: Optional[str] = ...,
        private_key: Optional[str] = ...,
        default_iss: Optional[str] = ...,
        default_aud: Optional[str] = ...,
        json_encoder: Any = ...,
        token_location: Tuple[str] = ...,
        access_token_expires: Union[timedelta, bool] = ...,
        refresh_token_expires: Union[timedelta, bool] = ...,
        algorithm: Optional[str] = ...,
        public_claim_namespace: Optional[str] = ...,
        private_claim_prefix: Optional[str] = ...,
        jwt_header_key: Optional[str] = ...,
        jwt_header_prefix: Optional[str] = ...,
        use_acl: Optional[str] = ...,
        acl_claim: Optional[str] = ...,
        use_blacklist: Optional[str] = ...,
        blacklist_class: Optional[Type[BlacklistABC]] = ...,
    ) -> None: ...
